@@ -651,7 +651,7 @@ int mainloop(void) {
 				mfs_log(MFSLOG_SYSLOG,MFSLOG_WARNING,"poll error: %s",strerr(errno));
 				break;
 			}
-		} else {
+		} else if (i>0) {
 			if ((pdesc[0].revents)&POLLIN) {
 				uint8_t sigid;
 				if (read(signalpipe[0],&sigid,1)==1) {
@@ -662,7 +662,9 @@ int mainloop(void) {
 						mfs_log(MFSLOG_SYSLOG,MFSLOG_INFO,"reloading config files");
 						r = 1;
 					} else if (sigid=='\003') {
+#ifndef SILENT_SIGCHLD
 						mfs_log(MFSLOG_SYSLOG,MFSLOG_INFO,"child finished");
+#endif
 						r = 2;
 					} else if (sigid=='\004') {
 						mfs_log(MFSLOG_SYSLOG,MFSLOG_INFO,"log extra info");
@@ -1612,6 +1614,46 @@ int main(int argc,char **argv) {
 #endif
 		nicelevel = cfg_getint32("NICE_LEVEL",-19);
 		setpriority(PRIO_PROCESS,getpid(),nicelevel);
+
+#if defined(__linux__) && defined(OOM_ADJUSTABLE)
+		if (cfg_getuint8("DISABLE_OOM_KILLER",1)==1) {
+			FILE *oomfd;
+			int oomdis;
+			oomdis = 0;
+#  if defined(OOM_SCORE_ADJ_MIN)
+			oomfd = fopen("/proc/self/oom_score_adj","w");
+			if (oomfd!=NULL) {
+				fprintf(oomfd,"%d\n",OOM_SCORE_ADJ_MIN);
+				if (fclose(oomfd)>=0) {
+					oomdis = 1;
+				}
+#    if defined(OOM_DISABLE)
+			} else {
+				oomfd = fopen("/proc/self/oom_adj","w");
+				if (oomfd!=NULL) {
+					fprintf(oomfd,"%d\n",OOM_DISABLE);
+					if (fclose(oomfd)>=0) {
+						oomdis = 1;
+					}
+				}
+#    endif
+			}
+#  elif defined(OOM_DISABLE)
+			oomfd = fopen("/proc/self/oom_adj","w");
+			if (oomfd!=NULL) {
+				fprintf(oomfd,"%d\n",OOM_DISABLE);
+				if (fclose(oomfd)>=0) {
+					oomdis = 1;
+				}
+			}
+#  endif
+			if (oomdis) {
+				mfs_log(MFSLOG_SYSLOG,MFSLOG_INFO,"out of memory killer disabled");
+			} else {
+				mfs_log(MFSLOG_SYSLOG,MFSLOG_WARNING,"can't disable out of memory killer");
+			}
+		}
+#endif
 	}
 
 #ifdef USE_IONICE
@@ -1741,42 +1783,6 @@ int main(int argc,char **argv) {
 	}
 #endif /* glibc malloc tuning */
 
-#if defined(__linux__) && defined(OOM_ADJUSTABLE)
-	if (cfg_getuint8("DISABLE_OOM_KILLER",1)==1) {
-		FILE *oomfd;
-		int oomdis;
-		oomdis = 0;
-#  if defined(OOM_SCORE_ADJ_MIN)
-		oomfd = fopen("/proc/self/oom_score_adj","w");
-		if (oomfd!=NULL) {
-			fprintf(oomfd,"%d\n",OOM_SCORE_ADJ_MIN);
-			fclose(oomfd);
-			oomdis = 1;
-#    if defined(OOM_DISABLE)
-		} else {
-			oomfd = fopen("/proc/self/oom_adj","w");
-			if (oomfd!=NULL) {
-				fprintf(oomfd,"%d\n",OOM_DISABLE);
-				fclose(oomfd);
-				oomdis = 1;
-			}
-#    endif
-		}
-#  elif defined(OOM_DISABLE)
-		oomfd = fopen("/proc/self/oom_adj","w");
-		if (oomfd!=NULL) {
-			fprintf(oomfd,"%d\n",OOM_DISABLE);
-			fclose(oomfd);
-			oomdis = 1;
-		}
-#  endif
-		if (oomdis) {
-			mfs_log(MFSLOG_SYSLOG,MFSLOG_INFO,"out of memory killer disabled");
-		} else {
-			mfs_log(MFSLOG_SYSLOG,MFSLOG_WARNING,"can't disable out of memory killer");
-		}
-	}
-#endif
 
 	mfs_log(MFSLOG_SYSLOG,MFSLOG_INFO,"monotonic clock function: %s",monotonic_method());
 	mfs_log(MFSLOG_SYSLOG,MFSLOG_INFO,"monotonic clock speed: %"PRIu32" ops / 10 mili seconds",monotonic_speed());
