@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2024 Jakub Kruszona-Zawadzki, Saglabs SA
+ * Copyright (C) 2025 Jakub Kruszona-Zawadzki, Saglabs SA
  * 
  * This file is part of MooseFS.
  * 
@@ -97,8 +97,8 @@ void sessions_datafill(uint8_t *ptr,uint8_t vmode);
 uint8_t sessions_force_remove(uint32_t sessionid);
 //uint32_t sessions_datasize(void *vsesdata,uint8_t vmode);
 //uint32_t sessions_datafill(uint8_t *ptr,void *vsesdata,uint8_t vmode);
-void* sessions_new_session(uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint8_t mingoal,uint8_t maxgoal,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng);
-uint32_t sessions_chg_session(void *vsesdata,uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint8_t mingoal,uint8_t maxgoal,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng);
+void* sessions_new_session(uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint16_t sclassgroups,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng);
+uint32_t sessions_chg_session(void *vsesdata,uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint16_t sclassgroups,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng);
 void sessions_sync_open_files(void *vsesdata,const uint8_t *ptr,uint32_t inodecnt);
 uint32_t sessions_get_id(void *vsesdata);
 uint64_t sessions_get_exportscsum(void *vsesdata);
@@ -108,7 +108,7 @@ uint32_t sessions_get_sesflags(void *vsesdata);
 uint16_t sessions_get_umask(void *vsesdata);
 uint32_t sessions_get_disables(void *vsesdata);
 uint8_t sessions_is_root_remapped(void *vsesdata);
-uint8_t sessions_check_goal(void *vsesdata,uint8_t smode,uint8_t goal);
+uint8_t sessions_check_sclass(void *vsesdata,uint8_t smode,uint8_t sclassid);
 uint8_t sessions_check_trashretention(void *vsesdata,uint8_t smode,uint32_t trashretention);
 void sessions_inc_stats(void *vsesdata,uint8_t statid);
 void sessions_add_stats(void *vsesdata,uint8_t statid,uint64_t value);
@@ -117,8 +117,8 @@ void sessions_ugid_remap(void *vsesdata,uint32_t *auid,uint32_t *agid);
 void sessions_cleanup(void);
 int sessions_init(void);
 
-uint8_t sessions_mr_seschanged(uint32_t sessionid,uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint8_t mingoal,uint8_t maxgoal,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng);
-uint8_t sessions_mr_sesadd(uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint8_t mingoal,uint8_t maxgoal,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng,uint32_t sessionid);
+uint8_t sessions_mr_seschanged(uint32_t sessionid,uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint16_t sclassgroups,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng);
+uint8_t sessions_mr_sesadd(uint64_t exportscsum,uint32_t rootinode,uint8_t sesflags,uint16_t umaskval,uint32_t rootuid,uint32_t rootgid,uint32_t mapalluid,uint32_t mapallgid,uint16_t sclassgroups,uint32_t mintrashretention,uint32_t maxtrashretention,uint32_t disables,uint32_t peerip,const uint8_t *info,uint32_t ileng,uint32_t sessionid);
 uint8_t sessions_mr_sesdel(uint32_t sessionid);
 uint8_t sessions_mr_connected(uint32_t sessionid);
 uint8_t sessions_mr_disconnected(uint32_t sessionid,uint32_t disctime);
