@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2025 Jakub Kruszona-Zawadzki, Saglabs SA
+ * 
+ * This file is part of MooseFS.
+ * 
+ * MooseFS is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, version 2 (only).
+ * 
+ * MooseFS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with MooseFS; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1301, USA
+ * or visit http://www.gnu.org/licenses/gpl-2.0.html
+ */
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -127,6 +147,7 @@ int scan_sclass(FILE *fd,metasection *sdata) {
 	uint8_t ec_is_turned_on;
 	uint8_t ogroup;
 	uint8_t nleng;
+	uint8_t dleng;
 	uint16_t sclassid;
 	uint8_t createcnt;
 	uint8_t keepcnt;
@@ -147,7 +168,7 @@ int scan_sclass(FILE *fd,metasection *sdata) {
 		sclass_arch_factor[sclassid]=(sclassid<10)?(8*sclassid):0;
 	}
 
-	if (sdata->mver>0x1B) {
+	if (sdata->mver>0x1C) {
 		fprintf(stderr,"unsupported sclass format\n");
 		return -1;
 	}
@@ -169,7 +190,7 @@ int scan_sclass(FILE *fd,metasection *sdata) {
 		rptr = buff;
 		exprsize = get16bit(&rptr);
 		ec_is_turned_on = get8bit(&rptr);
-		l = (sdata->mver>=0x1B)?42:(sdata->mver>=0x1A)?38:(sdata->mver>=0x18)?30:29;
+		l = (sdata->mver>=0x1C)?48:(sdata->mver>=0x1B)?42:(sdata->mver>=0x1A)?38:(sdata->mver>=0x18)?30:29;
 		while (1) {
 			if (fread(buff,1,2,fd)!=2) {
 				fprintf(stderr,"loading labelset: read error\n");
@@ -186,6 +207,12 @@ int scan_sclass(FILE *fd,metasection *sdata) {
 			}
 			rptr = buff;
 			nleng = get8bit(&rptr);
+			if (sdata->mver>=0x1C) {
+				dleng = get8bit(&rptr);
+				rptr += 5; // skip priority,export_group
+			} else {
+				dleng = 0;
+			}
 			rptr += 6; // skip admin_only,create_mode,arch_delay,min_trashretention
 			if (sdata->mver>=0x1A) {
 				rptr += 8; // skip arch_min_size
@@ -218,6 +245,9 @@ int scan_sclass(FILE *fd,metasection *sdata) {
 			if (nleng>0) {
 				fseeko(fd,nleng,SEEK_CUR);
 			}
+			if (dleng>0) {
+				fseeko(fd,dleng,SEEK_CUR);
+			}
 			if (sdata->mver<0x19) {
 				if (archec>1) {
 					archec--;
